/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: physics.h
*/

#ifndef DOBJECT_H
#define DOBJECT_H


/* Definition of dynamic object class */
class dobject_class 
{
     friend class world_class;
 
  public:
    
    dobject_class( void );    
    dobject_class( vector p, vector v );    // Init with staring pos and velocity

    ~dobject_class();         
  
    // Calculate all constants
    void update_const( void );

    // Reset dobject variables
    void init( void );
        
    void set_base_const( void );
  
    void set_face_mass(unsigned int &i, mass_type *mass_ptr, float tx_x, float tx_y);    

    void setup_face_masses( void );
    
    // Alloc memory for dobject
    void get_memory(void);    

    // Free all memory
    void free_memory(void);
       
    // Cpy data into graphics engine
    int copy_face_data( void );
    int copy_line_data( void );
    
    void make_xy_faces( unsigned int &i, float sub_x, float sub_y, float sub_z);
    void make_xz_faces( unsigned int &i, float sub_x, float sub_y, float sub_z);
    void make_yz_faces( unsigned int &i, float sub_x, float sub_y, float sub_z);
            
    // Calcuate tension along spring
    void get_tension(tension_type &t, state delta, double base);    

    state get_state(int ix, int iy, int iz);
    
    // Calculate force from delta
    double get_force(vector delta, double len, double rest);
  
    // Calculate force acting on a point
    vector calc_point_force( unsigned int ix, unsigned int iy, unsigned int iz);  
    
    // Range check point
    void clip( mass_type &p );
  
    void calc_tensions(void);           // Calculate all tensions
    void calc_torques(void);            // Calculate all torques
    void calc_forces( void );           // Calculate all forces
    
    void intergrate_step( void );       // Recalculation for integration
  
    // Determine if point should be updated
    bool exclude_point(int ix, int iy, int iz);
 
    // Function for numerical intergration
    state dobject_class::f(mass_type &p);

    // Preform numerical  integration
    void intergrate_rk4( void );               

    void update_state(void);

    // Preform total physics update
    void physics_update( void )  ;
  
    // Get closested point to screen cords
    void get_close_point(double wx, double wy, int &ix, int &iy);
  
     void render( void );
  
    // Increment point num accessors
    void inc_points_x ( void );
    void inc_points_y ( void );
    void inc_points_z ( void );
    
    // Decrement point num accessors
    void dec_points_x ( void );
    void dec_points_y ( void );
    void dec_points_z ( void );
  

    void inc_mass ( void );     // Increase mass
    void dec_mass ( void );     // Decrease mass
  
    void inc_spring_k ( void ); // Increase spring constant
    void dec_spring_k ( void ); // Decrease spring constant
    
    void inc_torque_k ( void ); // Increase torque constant
    void dec_torque_k ( void ); // Decrease torque constant

    void inc_drag ( void );     // Increase torque constant
    void dec_drag ( void );     // Decrease torque constant    
    
  
    // Return base index for xyz index      
    int get_index(int ix, int iy, int iz);

    double get_total_mass ( void ) { return total_mass; };
    double get_spring_k ( void ) { return base_spring_k ; };
    double get_torque_k ( void ) { return base_torque_k; };
    double get_drag ( void ) { return base_drag; };

    unsigned int get_num_triangles ( void ) { return num_triangles; };

    intvector get_points ( void ) { return num_points; };

    LPDIRECT3DVERTEXBUFFER9 dx_vbuff;       // Vertex Buffer

  private:

    unsigned int total_points;              // Total number of points

    unsigned int num_lines;                 // Number of lines for faces
    unsigned int num_triangles;             // Number of triangles for faces
    unsigned int num_face_points;
    
    vertex *vert_data;                 // Internal vertex buffer for copy

    face_mass_type *face_masses;            // Pointer to masses on face

    mass_type *points;                      // All masses in object
  
    int test_pt_x, test_pt_y, test_pt_z;    // Selected point in object

    intvector num_points;  // Number of masses in each direction

    quaternion orientation; // Current orientation in world space

    vector ang;             // Angle in world space, only used at startup
    vector ang_vel;        // Local angle velocity of object
    
    vector point_size;     // Size of mass in 3d
    vector rest_len;       // Resting length of spring elements

    state cur_state;          // Current Position and Velocity for entire object
    
    vector size;           // Size in three dimensions    

    bool apply_ground_friction; // True if touching the ground
    

    double base_dampening;
    double dampening;       // Dampening constant for mass to spring interaction

    double snap_percent;    // Percent spring has to bend to snap
    double base_drag;
    double base_spring_k;   // Spring constant
    double base_torque_k;   // Torque constant
    double torque_k;        // Torque constant per point
    double spring_k;        // Spring  constant per point
    double drag;            // Internal drag
    
    double var_base;        // Base for pos variance
    double total_mass;      // Mass of entire object
    double point_mass;      // Mass of each point
        
    double mass_radius;        
        
    // Structural tensions in x y and z directions
    tension_type *tension_x, *tension_y, *tension_z;
       
    bool is_projectile;     // True if this is a projectile
    
    dobject_class *next;    // Pointer to next object    
};


#endif

