/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: math3d.h
*/


#ifndef MATH_H
#define MATH_H

#define epsilon 0.000001                 // Lower limit for divisor

#define zero (vector(0.0, 0.0, 0.0))

#define unit_x (vector(1.0, 0.0, 0.0))
#define unit_y (vector(0.0, 1.0, 0.0))
#define unit_z (vector(0.0, 0.0, 1.0))


/* Vector operations */

double dot( vector p1, vector p2 );
vector cross( vector p1, vector p2 );

vector operator* ( double s, vector p );
vector operator* ( vector p, double s );
vector operator/ ( vector p, double s );
vector operator- ( vector p1, vector p2 );
vector operator+ ( vector p1, vector p2 );

vector operator+= ( vector &p1, vector p2 );
vector operator-= ( vector &p1, vector p2 );


/* State operations */

state operator* ( double s, state p );
state operator/ ( state p, double s );
state operator+ ( state s1, state s2 );
state operator- ( state s1, state s2 );

state operator+= ( state &s1, state s2 );
state operator-= ( state &s1, state s2 );


/* Quaternion operations */

quaternion operator+ ( quaternion q1, quaternion q2 );
quaternion operator+= ( quaternion &q1, quaternion q2 );

quaternion operator- ( quaternion q1, quaternion q2 );
quaternion operator-= ( quaternion &q1, quaternion q2 );

quaternion operator* ( quaternion q1, quaternion q2 );
quaternion operator* ( quaternion q, vector v );
quaternion operator* ( vector v, quaternion q );
quaternion operator* ( quaternion q, double s );

void quaternion_rotate_vector( quaternion q, vector &v );

quaternion quaternion_from_angles( vector ang );


#endif


