/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: types.h
*/



/* 3d vector type */
typedef struct vector
{
    vector(void) {x = 0; y = 0; z = 0;};
    vector(double vx, double vy, double vz) { x = vx; y = vy; z = vz;};

    double mag( void );
    void normalize(void);

    vector norm();

    double x,y,z;
};
/* End of vector */



/* 3d Position and velocity */
typedef struct state
{
    state(void) { pos = vector(0,0,0); vel = vector(0,0,0);};
    state(vector spos, vector svel) { pos = spos; vel = svel;};
    
    vector pos,vel;
};
/* End of State type */


/* Integer point type */
typedef struct intvector
{
    intvector(void) {};
    intvector(int vx, int vy, int vz) { x = vx; y = vy; z = vz;};
    
    int x, y, z;
};
/* End of inpint3d  */


/* Quaternion type */
typedef struct quaternion
{
    quaternion( void ) { v = vector(); w = 0; };
    quaternion( vector qv, double qw ) { v = qv; w = qw; };
    quaternion( double vx, double vy, double vz, double qw ) { v.x = vx; v.y=vy; v.z = vz; w = qw; };

    double mag( void );
    void normalize( void );

    quaternion conjugate( void );

    vector v;  // Vector part
    double w;   // Scalar part
};
/* End of Quaternion type */



/* Tension */
typedef struct tension_type
{
    vector force;               // Directinal force
    
    vector delta;               // Displament of spring
    vector norm_delta;          // Norm of displament of spring
    
    bool snapped;               // True if spring as snapped
};
/* End of tension type */


/* Mass in dobject */
typedef struct mass_type
{
    state cur_state;            // Current position and velocity
    vector force;               // Current force total   
    vector ext_force;           // External force
    vector world_pos;           // Position in world space

    bool fixed;                 // True if mass can not move
};
/* End of mass_type */


/* Structure for face masses. Used for faster rendering */
typedef struct face_mass_type
{
    mass_type *mass;            // Pointer to mass type

    double tx_x, tx_y;          // Texture coords
};
/* End of face_mass_type */


/* Custom Direct3d vertex type */
struct vertex
{
    vertex(){};
    vertex(vector p, float nu, float nv){x=p.x; y=p.y; z=p.z; u=nu; v=nv;};

    void vertex::set( face_mass_type &f );
    
    float x, y, z;      // Position in 3d
    float u, v;         // Texture coords
};
/* Endof vertex_type */



